@@ -20,7 +20,7 @@
 /*
  * Server side of the protocols.
  */
-/* $Id: protocol.c,v 1.79.2.2 2002/04/03 10:36:27 priikone Exp $ */
+/* $Id: protocol.c,v 1.86 2002/06/15 19:38:37 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -291,13 +291,14 @@ int silc_server_protocol_ke_set_keys(SilcServer server,
   idata->rekey->ske_group = silc_ske_group_get_number(group);
 
   /* Save the hash */
-  if (!silc_hash_alloc(hash->hash->name, &idata->hash)) {
+  if (!silc_hash_alloc(silc_hash_get_name(hash), &idata->hash)) {
     silc_cipher_free(idata->send_key);
     silc_cipher_free(idata->receive_key);
     silc_hmac_free(idata->hmac_send);
     silc_hmac_free(idata->hmac_receive);
     silc_free(conn_data);
-    SILC_LOG_ERROR(("Cannot allocate algorithm: %s", hash->hash->name));
+    SILC_LOG_ERROR(("Cannot allocate algorithm: %s", 
+		    silc_hash_get_name(hash)));
     return FALSE;
   }
 
@@ -314,7 +315,7 @@ int silc_server_protocol_ke_set_keys(SilcServer server,
 		 sock->hostname, sock->ip,
 		 idata->send_key->cipher->name,
 		 (char *)silc_hmac_get_name(idata->hmac_send),
-		 idata->hash->hash->name, 
+		 silc_hash_get_name(idata->hash),
 		 ske->prop->flags & SILC_SKE_SP_FLAG_PFS ? "PFS" : ""));
 
   return TRUE;
@@ -325,60 +326,36 @@ int silc_server_protocol_ke_set_keys(SilcServer server,
 SilcSKEStatus silc_ske_check_version(SilcSKE ske, unsigned char *version,
 				     SilcUInt32 len, void *context)
 {
-  SilcSKEStatus status = SILC_SKE_STATUS_OK;
-  char *cp;
-  int maj = 0, min = 0, build = 0, maj2 = 0, min2 = 0, build2 = 0;
+  SilcUInt32 l_protocol_version = 0, r_protocol_version = 0;
 
   SILC_LOG_INFO(("%s (%s) is version %s", ske->sock->hostname,
 		 ske->sock->ip, version));
 
-  /* Check for initial version string. Allowed "SILC-x.x-". More 
-     specific protocol version is checked later in session. */
-  if (!strstr(version, "SILC-"))
-    status = SILC_SKE_STATUS_BAD_VERSION;
-
-  /* Check software version */
-
-  cp = version + 9;
-  if (!cp)
-    status = SILC_SKE_STATUS_BAD_VERSION;
-
-  maj = atoi(cp);
-  cp = strchr(cp, '.');
-  if (cp) {
-    min = atoi(cp + 1);
-    cp++;
-  }
-  if (cp) {
-    cp = strchr(cp, '.');
-    if (cp)
-      build = atoi(cp + 1);
-  }
-
-  cp = silc_version_string + 9;
-  if (!cp)
-    status = SILC_SKE_STATUS_BAD_VERSION;
-
-  maj2 = atoi(cp);
-  cp = strchr(cp, '.');
-  if (cp) {
-    min2 = atoi(cp + 1);
-    cp++;
-  }
-  if (cp) {
-    cp = strchr(cp, '.');
-    if (cp)
-      build2 = atoi(cp + 1);
-  }
-
-  if (maj != maj2)
-    status = SILC_SKE_STATUS_BAD_VERSION;
-
-  if (status == SILC_SKE_STATUS_BAD_VERSION)
+  if (!silc_parse_version_string(version, &r_protocol_version, NULL, NULL,
+				 NULL, NULL)) {
     SILC_LOG_ERROR(("%s (%s) %s is not allowed/supported version", 
 		    ske->sock->hostname, ske->sock->ip, version));
+    return SILC_SKE_STATUS_BAD_VERSION;
+  }
 
-  return status;
+  if (!silc_parse_version_string(silc_version_string, 
+				 &l_protocol_version, NULL, NULL,
+				 NULL, NULL)) {
+    SILC_LOG_ERROR(("%s (%s) %s is not allowed/supported version", 
+		    ske->sock->hostname, ske->sock->ip, version));
+    return SILC_SKE_STATUS_BAD_VERSION;
+  }
+
+  /* If remote is too new, don't connect */
+  if (l_protocol_version < r_protocol_version) {
+    SILC_LOG_ERROR(("%s (%s) %s is not allowed/supported version", 
+		    ske->sock->hostname, ske->sock->ip, version));
+    return SILC_SKE_STATUS_BAD_VERSION;
+  }
+
+  ske->sock->version = r_protocol_version;
+
+  return SILC_SKE_STATUS_OK;
 }
 
 /* Callback that is called by the SKE to indicate that it is safe to
@@ -614,7 +591,7 @@ SILC_TASK_CALLBACK(silc_server_protocol_key_exchange)
        */
       SilcSKEKeyMaterial *keymat;
       int key_len = silc_cipher_get_key_len(ctx->ske->prop->cipher);
-      int hash_len = ctx->ske->prop->hash->hash->hash_len;
+      int hash_len = silc_hash_len(ctx->ske->prop->hash);
 
       /* Process the key material */
       keymat = silc_calloc(1, sizeof(*keymat));
@@ -714,10 +691,10 @@ SILC_TASK_CALLBACK(silc_server_protocol_key_exchange)
  */
 
 static int 
-silc_server_password_authentication(SilcServer server, char *remote_auth, 
-				    char *local_auth)
+silc_server_password_authentication(SilcServer server, char *local_auth, 
+				    char *remote_auth)
 {
-  if (!remote_auth || !local_auth)
+  if (!remote_auth || !local_auth || strlen(local_auth) != strlen(remote_auth))
     return FALSE;
 
   if (!memcmp(remote_auth, local_auth, strlen(local_auth)))
@@ -1260,7 +1237,7 @@ void silc_server_protocol_rekey_generate(SilcServer server,
   SilcIDListData idata = (SilcIDListData)ctx->sock->user_data;
   SilcSKEKeyMaterial *keymat;
   SilcUInt32 key_len = silc_cipher_get_key_len(idata->send_key);
-  SilcUInt32 hash_len = idata->hash->hash->hash_len;
+  SilcUInt32 hash_len = silc_hash_len(idata->hash);
 
   SILC_LOG_DEBUG(("Generating new %s session keys (no PFS)",
 		  send ? "sending" : "receiving"));
@@ -1289,7 +1266,7 @@ silc_server_protocol_rekey_generate_pfs(SilcServer server,
   SilcIDListData idata = (SilcIDListData)ctx->sock->user_data;
   SilcSKEKeyMaterial *keymat;
   SilcUInt32 key_len = silc_cipher_get_key_len(idata->send_key);
-  SilcUInt32 hash_len = idata->hash->hash->hash_len;
+  SilcUInt32 hash_len = silc_hash_len(idata->hash);
   unsigned char *tmpbuf;
   SilcUInt32 klen;
 
