@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: server_backup.c,v 1.18.2.1 2002/04/01 22:43:53 johnny Exp $ */
+/* $Id: server_backup.c,v 1.20 2002/05/30 13:54:42 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -523,13 +523,15 @@ SILC_TASK_CALLBACK(silc_server_backup_connect_to_router)
   SilcServerConnection sconn = (SilcServerConnection)context;
   SilcServer server = sconn->server;
   int sock;
+  const char *server_ip;
 
   SILC_LOG_DEBUG(("Connecting to router %s:%d", sconn->remote_host,
 		  sconn->remote_port));
 
   /* Connect to remote host */
-  sock = silc_net_create_connection(server->config->server_info->server_ip,
-				    sconn->remote_port,
+  server_ip = server->config->server_info->primary == NULL ? NULL :
+	  	server->config->server_info->primary->server_ip;
+  sock = silc_net_create_connection(server_ip, sconn->remote_port,
 				    sconn->remote_host);
   if (sock < 0) {
     silc_schedule_task_add(server->schedule, 0,
