@@ -29,8 +29,9 @@ typedef struct {
   SilcUInt32 my_servers;		  /* Locally connected servers */
   SilcUInt32 my_routers;		  /* Locally connected routers */
   SilcUInt32 my_channels;		  /* Locally created channels */
-  SilcUInt32 my_chanclients;	  /* Local clients on local channels */
-  SilcUInt32 my_aways;		  /* Local clients away (XXX) */
+  SilcUInt32 my_chanclients;	          /* Local clients on local channels */
+  SilcUInt32 my_aways;			  /* Local clients away (gone) */
+  SilcUInt32 my_detached;		  /* Local clients detached */
   SilcUInt32 my_server_ops;		  /* Local server operators */
   SilcUInt32 my_router_ops;		  /* Local router operators */
 
@@ -38,12 +39,14 @@ typedef struct {
   SilcUInt32 cell_clients;		  /* All clients in cell */
   SilcUInt32 cell_servers;		  /* All servers in cell */
   SilcUInt32 cell_channels;		  /* All channels in cell */
-  SilcUInt32 cell_chanclients;	  /* All clients on cell's channels */
-  SilcUInt32 clients;		  /* All clients */
-  SilcUInt32 servers;		  /* All servers */
-  SilcUInt32 routers;		  /* All routers */
-  SilcUInt32 channels;		  /* All channels */
+  SilcUInt32 cell_chanclients;		  /* All clients on cell's channels */
+  SilcUInt32 clients;			  /* All clients */
+  SilcUInt32 servers;			  /* All servers */
+  SilcUInt32 routers;			  /* All routers */
+  SilcUInt32 channels;			  /* All channels */
   SilcUInt32 chanclients;		  /* All clients on channels */
+  SilcUInt32 aways;			  /* All clients away (gone) */
+  SilcUInt32 detached;		          /* All clients detached */
   SilcUInt32 server_ops;		  /* All server operators */
   SilcUInt32 router_ops;		  /* All router operators */
 
@@ -53,7 +56,7 @@ typedef struct {
   SilcUInt32 auth_attempts;		  /* Authentication attempts */
   SilcUInt32 auth_failures;		  /* Authentication failures */
   SilcUInt32 packets_sent;		  /* Sent packets */
-  SilcUInt32 packets_received;	  /* Received packets */
+  SilcUInt32 packets_received;		  /* Received packets */
 } SilcServerStatistics;
 
 /*
@@ -82,8 +85,8 @@ struct SilcServerStruct {
   bool backup_router;		     /* TRUE if this is backup router */
   bool backup_primary;		     /* TRUE if we've switched our primary
 				        router to a backup router. */
-  SilcServerConnection router_conn; /* non-NULL when connecting to the
-				       primary router, and NULL otherwise. */
+  SilcServerConnection router_conn;  /* non-NULL when connecting to the
+					primary router, and NULL otherwise. */
 
   /* Current command identifier, 0 not used */
   SilcUInt16 cmd_ident;
@@ -177,6 +180,7 @@ do {						\
       server->stat.my_ ## type ## _ops--;	\
     if (server->server_type == SILC_ROUTER)	\
       server->stat. type ## _ops--;		\
+    (c)->mode &= ~(mod);			\
   }						\
 } while(0)
 
