@@ -16,11 +16,13 @@
   GNU General Public License for more details.
 
 */
-/* $Id: server_util.c,v 1.26.2.2 2002/04/01 22:43:53 johnny Exp $ */
+/* $Id: server_util.c,v 1.44 2002/06/15 16:06:55 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
 
+extern char *server_version;
+
 /* Removes the client from channels and possibly removes the channels
    as well.  After removing those channels that exist, their channel
    keys are regnerated. This is called only by the function
@@ -49,27 +51,18 @@ static void silc_server_remove_clients_channels(SilcServer server,
   while (silc_hash_table_get(&htl, NULL, (void **)&chl)) {
     channel = chl->channel;
 
-    /* Remove channel from client's channel list */
-    silc_hash_table_del(client->channels, channel);
-
-    /* Remove channel if there is no users anymore */
+    /* Remove channel if this is last client leaving the channel, unless
+       the channel is permanent. */
     if (server->server_type == SILC_ROUTER &&
 	silc_hash_table_count(channel->user_list) < 2) {
-
       if (silc_hash_table_find(channels, channel, NULL, NULL))
 	silc_hash_table_del(channels, channel);
-
-      if (channel->rekey)
-	silc_schedule_task_del_by_context(server->schedule, channel->rekey);
-
-      if (silc_idlist_del_channel(server->local_list, channel))
-	server->stat.my_channels--;
-      else 
-        silc_idlist_del_channel(server->global_list, channel);
+      silc_schedule_task_del_by_context(server->schedule, channel->rekey);
+      silc_server_channel_delete(server, channel);
       continue;
     }
 
-    /* Remove client from channel's client list */
+    silc_hash_table_del(client->channels, channel);
     silc_hash_table_del(channel->user_list, chl->client);
     channel->user_count--;
 
@@ -80,42 +73,24 @@ static void silc_server_remove_clients_channels(SilcServer server,
       channel->global_users = FALSE;
 
     silc_free(chl);
-    server->stat.my_chanclients--;
+
+    /* Update statistics */
+    if (client->connection)
+      server->stat.my_chanclients--;
+    if (server->server_type == SILC_ROUTER) {
+      server->stat.cell_chanclients--;
+      server->stat.chanclients--;
+    }
 
     /* If there is not at least one local user on the channel then we don't
-       need the channel entry anymore, we can remove it safely. */
+       need the channel entry anymore, we can remove it safely, unless the
+       channel is permanent channel */
     if (server->server_type != SILC_ROUTER &&
 	!silc_server_channel_has_local(channel)) {
-
       if (silc_hash_table_find(channels, channel, NULL, NULL))
 	silc_hash_table_del(channels, channel);
-
-      if (channel->rekey)
-	silc_schedule_task_del_by_context(server->schedule, channel->rekey);
-
-      if (channel->founder_key) {
-	/* The founder auth data exists, do not remove the channel entry */
-	SilcChannelClientEntry chl2;
-	SilcHashTableList htl2;
-
-	channel->disabled = TRUE;
-
-	silc_hash_table_list(channel->user_list, &htl2);
-	while (silc_hash_table_get(&htl2, NULL, (void **)&chl2)) {
-	  silc_hash_table_del(chl2->client->channels, channel);
-	  silc_hash_table_del(channel->user_list, chl2->client);
-	  channel->user_count--;
-	  silc_free(chl2);
-	}
-	silc_hash_table_list_reset(&htl2);
-	continue;
-      }
-
-      /* Remove the channel entry */
-      if (silc_idlist_del_channel(server->local_list, channel))
-	server->stat.my_channels--;
-      else 
-        silc_idlist_del_channel(server->global_list, channel);
+      silc_schedule_task_del_by_context(server->schedule, channel->rekey);
+      silc_server_channel_delete(server, channel);
       continue;
     }
 
@@ -125,7 +100,6 @@ static void silc_server_remove_clients_channels(SilcServer server,
       silc_hash_table_add(channels, channel, channel);
   }
   silc_hash_table_list_reset(&htl);
-
   silc_buffer_free(clidp);
 }
 
@@ -216,7 +190,7 @@ bool silc_server_remove_clients_by_server(SilcServer server,
 
 	/* Update statistics */
 	server->stat.clients--;
-	if (server->server_type == SILC_ROUTER)
+	if (server->stat.cell_clients)
 	  server->stat.cell_clients--;
 	SILC_OPER_STATS_UPDATE(client, server, SILC_UMODE_SERVER_OPERATOR);
 	SILC_OPER_STATS_UPDATE(client, router, SILC_UMODE_ROUTER_OPERATOR);
@@ -227,6 +201,8 @@ bool silc_server_remove_clients_by_server(SilcServer server,
 	  client->data.status &= ~SILC_IDLIST_STATUS_REGISTERED;
 	  id_cache->expire = SILC_ID_CACHE_EXPIRE_DEF;
 	} else {
+	  /* Remove this client from watcher list if it is */
+	  silc_server_del_from_watcher_list(server, client);
 	  silc_idlist_del_client(server->local_list, client);
 	}
 
@@ -280,7 +256,7 @@ bool silc_server_remove_clients_by_server(SilcServer server,
 
 	/* Update statistics */
 	server->stat.clients--;
-	if (server->server_type == SILC_ROUTER)
+	if (server->stat.cell_clients)
 	  server->stat.cell_clients--;
 	SILC_OPER_STATS_UPDATE(client, server, SILC_UMODE_SERVER_OPERATOR);
 	SILC_OPER_STATS_UPDATE(client, router, SILC_UMODE_ROUTER_OPERATOR);
@@ -655,7 +631,7 @@ void silc_server_remove_channels_by_server(SilcServer server,
   SilcIDCacheEntry id_cache = NULL;
   SilcChannelEntry channel = NULL;
 
-  SILC_LOG_DEBUG(("Start"));
+  SILC_LOG_DEBUG(("Removing channels by server"));
 
   if (silc_idcache_get_all(server->global_list->channels, &list)) {
     if (silc_idcache_list_first(list, &id_cache)) {
@@ -681,7 +657,7 @@ void silc_server_update_channels_by_server(SilcServer server,
   SilcIDCacheEntry id_cache = NULL;
   SilcChannelEntry channel = NULL;
 
-  SILC_LOG_DEBUG(("Start"));
+  SILC_LOG_DEBUG(("Updating channels by server"));
 
   if (silc_idcache_get_all(server->global_list->channels, &list)) {
     if (silc_idcache_list_first(list, &id_cache)) {
@@ -737,6 +713,68 @@ bool silc_server_channel_has_local(SilcChannelEntry channel)
   return FALSE;
 }
 
+/* This function removes the channel and all users on the channel, unless
+   the channel is permanent.  In this case the channel is disabled but all
+   users are removed from the channel.  Returns TRUE if the channel is
+   destroyed totally, and FALSE if it is permanent and remains. */
+
+bool silc_server_channel_delete(SilcServer server,
+				SilcChannelEntry channel)
+{
+  SilcChannelClientEntry chl;
+  SilcHashTableList htl;
+  bool delchan = !(channel->mode & SILC_CHANNEL_MODE_FOUNDER_AUTH);
+
+  if (delchan) {
+    SILC_LOG_DEBUG(("Deleting %s channel", channel->channel_name));
+
+    /* Update statistics */
+    if (server->server_type == SILC_ROUTER)
+      server->stat.chanclients -= channel->user_count;
+
+    /* Totally delete the channel and all users on the channel. The
+       users are deleted automatically in silc_idlist_del_channel. */
+    silc_schedule_task_del_by_context(server->schedule, channel->rekey);
+    if (silc_idlist_del_channel(server->local_list, channel)) {
+      server->stat.my_channels--;
+      if (server->server_type == SILC_ROUTER) {
+	server->stat.channels--;
+	server->stat.cell_channels--;
+      }
+    } else {
+      if (silc_idlist_del_channel(server->global_list, channel))
+	if (server->server_type == SILC_ROUTER)
+	  server->stat.channels--;
+    }
+
+    return FALSE;
+  }
+
+  /* Channel is permanent, do not remove it, remove only users */
+  channel->disabled = TRUE;
+  silc_hash_table_list(channel->user_list, &htl);
+  while (silc_hash_table_get(&htl, NULL, (void *)&chl)) {
+    silc_hash_table_del(chl->client->channels, channel);
+    silc_hash_table_del(channel->user_list, chl->client);
+    channel->user_count--;
+
+    /* Update statistics */
+    if (chl->client->connection)
+      server->stat.my_chanclients--;
+    if (server->server_type == SILC_ROUTER) {
+      server->stat.cell_chanclients--;
+      server->stat.chanclients--;
+    }
+
+    silc_free(chl);
+  }
+  silc_hash_table_list_reset(&htl);
+
+  SILC_LOG_DEBUG(("Channel %s remains", channel->channel_name));
+
+  return TRUE;
+}
+
 /* Returns TRUE if the given client is on the channel.  FALSE if not. 
    This works because we assure that the user list on the channel is
    always in up to date thus we can only check the channel list from 
@@ -924,13 +962,15 @@ bool silc_server_connection_allowed(SilcServer server,
   if (ske && silc_ske_parse_version(ske, &r_protocol_version, NULL,
 				    &r_software_version, NULL,
 				    &r_vendor_version)) {
+    sock->version = r_protocol_version;
+
     /* Match protocol version */
     if (l_protocol_version && r_protocol_version &&
 	r_protocol_version < l_protocol_version) {
       SILC_LOG_INFO(("Connection %s (%s) is too old version",
 		     sock->hostname, sock->ip));
       silc_server_disconnect_remote(server, sock, 
-				    "Server closed connection: "
+				    SILC_STATUS_ERR_BAD_VERSION,
 				    "You support too old protocol version");
       return FALSE;
     }
@@ -941,7 +981,7 @@ bool silc_server_connection_allowed(SilcServer server,
       SILC_LOG_INFO(("Connection %s (%s) is too old version",
 		     sock->hostname, sock->ip));
       silc_server_disconnect_remote(server, sock, 
-				    "Server closed connection: "
+				    SILC_STATUS_ERR_BAD_VERSION,
 				    "You support too old software version");
       return FALSE;
     }
@@ -952,7 +992,7 @@ bool silc_server_connection_allowed(SilcServer server,
       SILC_LOG_INFO(("Connection %s (%s) is unsupported version",
 		     sock->hostname, sock->ip));
       silc_server_disconnect_remote(server, sock, 
-				    "Server closed connection: "
+				    SILC_STATUS_ERR_BAD_VERSION,
 				    "Your software is not supported");
       return FALSE;
     }
@@ -970,7 +1010,7 @@ bool silc_server_connection_allowed(SilcServer server,
     SILC_LOG_INFO(("Server is full, closing %s (%s) connection",
 		   sock->hostname, sock->ip));
     silc_server_disconnect_remote(server, sock, 
-				  "Server closed connection: "
+				  SILC_STATUS_ERR_RESOURCE_LIMIT,
 				  "Server is full, try again later");
     return FALSE;
   }
@@ -979,7 +1019,7 @@ bool silc_server_connection_allowed(SilcServer server,
     SILC_LOG_INFO(("Too many connections from %s (%s), closing connection",
 		   sock->hostname, sock->ip));
     silc_server_disconnect_remote(server, sock, 
-				  "Server closed connection: "
+				  SILC_STATUS_ERR_RESOURCE_LIMIT,
 				  "Too many connections from your host");
     return FALSE;
   }
@@ -1006,14 +1046,17 @@ bool silc_server_check_cmode_rights(SilcServer server,
   if (is_op && is_fo)
     return TRUE;
 
+  /* Founder implies operator */
+  if (is_fo)
+    is_op = TRUE;
+
   /* We know that client is channel operator, check that they are not
      changing anything that requires channel founder rights. Rest of the
      modes are available automatically for channel operator. */
 
   if (mode & SILC_CHANNEL_MODE_PRIVKEY) {
-    if (!(channel->mode & SILC_CHANNEL_MODE_PRIVKEY))
-      if (is_op && !is_fo)
-	return FALSE;
+    if (is_op && !is_fo)
+      return FALSE;
   } else {
     if (channel->mode & SILC_CHANNEL_MODE_PRIVKEY) {
       if (is_op && !is_fo)
@@ -1022,9 +1065,8 @@ bool silc_server_check_cmode_rights(SilcServer server,
   }
   
   if (mode & SILC_CHANNEL_MODE_PASSPHRASE) {
-    if (!(channel->mode & SILC_CHANNEL_MODE_PASSPHRASE))
-      if (is_op && !is_fo)
-	return FALSE;
+    if (is_op && !is_fo)
+      return FALSE;
   } else {
     if (channel->mode & SILC_CHANNEL_MODE_PASSPHRASE) {
       if (is_op && !is_fo)
@@ -1033,9 +1075,8 @@ bool silc_server_check_cmode_rights(SilcServer server,
   }
 
   if (mode & SILC_CHANNEL_MODE_CIPHER) {
-    if (!(channel->mode & SILC_CHANNEL_MODE_CIPHER))
-      if (is_op && !is_fo)
-	return FALSE;
+    if (is_op && !is_fo)
+      return FALSE;
   } else {
     if (channel->mode & SILC_CHANNEL_MODE_CIPHER) {
       if (is_op && !is_fo)
@@ -1044,9 +1085,8 @@ bool silc_server_check_cmode_rights(SilcServer server,
   }
   
   if (mode & SILC_CHANNEL_MODE_FOUNDER_AUTH) {
-    if (!(channel->mode & SILC_CHANNEL_MODE_FOUNDER_AUTH))
-      if (is_op && !is_fo)
-	return FALSE;
+    if (is_op && !is_fo)
+      return FALSE;
   } else {
     if (channel->mode & SILC_CHANNEL_MODE_FOUNDER_AUTH) {
       if (is_op && !is_fo)
@@ -1054,6 +1094,26 @@ bool silc_server_check_cmode_rights(SilcServer server,
     }
   }
   
+  if (mode & SILC_CHANNEL_MODE_SILENCE_USERS) {
+    if (is_op && !is_fo)
+      return FALSE;
+  } else {
+    if (channel->mode & SILC_CHANNEL_MODE_SILENCE_USERS) {
+      if (is_op && !is_fo)
+	return FALSE;
+    }
+  }
+  
+  if (mode & SILC_CHANNEL_MODE_SILENCE_OPERS) {
+    if (is_op && !is_fo)
+      return FALSE;
+  } else {
+    if (channel->mode & SILC_CHANNEL_MODE_SILENCE_OPERS) {
+      if (is_op && !is_fo)
+	return FALSE;
+    }
+  }
+  
   return TRUE;
 }
 
@@ -1093,3 +1153,299 @@ bool silc_server_check_umode_rights(SilcServer server,
 
   return TRUE;
 }
+
+/* This function is used to send the notify packets and motd to the
+   incoming client connection. */
+
+void silc_server_send_connect_notifys(SilcServer server,
+				      SilcSocketConnection sock,
+				      SilcClientEntry client)
+{
+  SilcIDListData idata = (SilcIDListData)client;
+
+  /* Send some nice info to the client */
+  SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			  ("Welcome to the SILC Network %s",
+			   client->username));
+  SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			  ("Your host is %s, running version %s",
+			   server->server_name, server_version));
+
+  if (server->server_type == SILC_ROUTER) {
+    SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			    ("There are %d clients, %d servers and %d "
+			     "routers in SILC Network",
+			     server->stat.clients, server->stat.servers + 1,
+			     server->stat.routers));
+  } else {
+    if (server->stat.clients && server->stat.servers + 1)
+      SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			      ("There are %d clients, %d servers and %d "
+			       "routers in SILC Network",
+			       server->stat.clients, server->stat.servers + 1,
+			       (server->standalone ? 0 :
+				!server->stat.routers ? 1 :
+				server->stat.routers)));
+  }
+
+  if (server->stat.cell_clients && server->stat.cell_servers + 1)
+    SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			    ("There are %d clients on %d server in our cell",
+			     server->stat.cell_clients,
+			     server->stat.cell_servers + 1));
+  if (server->server_type == SILC_ROUTER) {
+    SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			    ("I have %d clients, %d channels, %d servers and "
+			     "%d routers",
+			     server->stat.my_clients, 
+			     server->stat.my_channels,
+			     server->stat.my_servers,
+			     server->stat.my_routers));
+  } else {
+    SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			    ("I have %d clients and %d channels formed",
+			     server->stat.my_clients,
+			     server->stat.my_channels));
+  }
+
+  if (server->stat.server_ops || server->stat.router_ops)
+    SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			    ("There are %d server operators and %d router "
+			     "operators online",
+			     server->stat.server_ops,
+			     server->stat.router_ops));
+  if (server->stat.my_router_ops + server->stat.my_server_ops)
+    SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			    ("I have %d operators online",
+			     server->stat.my_router_ops +
+			     server->stat.my_server_ops));
+
+  SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			  ("Your connection is secured with %s cipher, "
+			   "key length %d bits",
+			   idata->send_key->cipher->name,
+			   idata->send_key->cipher->key_len));
+  SILC_SERVER_SEND_NOTIFY(server, sock, SILC_NOTIFY_TYPE_NONE,
+			  ("Your current nickname is %s",
+			   client->nickname));
+
+  /* Send motd */
+  silc_server_send_motd(server, sock);
+}
+
+/* Kill the client indicated by `remote_client' sending KILLED notify
+   to the client, to all channels client has joined and to primary
+   router if needed.  The killed client is also removed from all channels. */
+
+void silc_server_kill_client(SilcServer server,
+			     SilcClientEntry remote_client,
+			     const char *comment,
+			     void *killer_id,
+			     SilcIdType killer_id_type)
+{
+  SilcBuffer killed, killer;
+
+  SILC_LOG_DEBUG(("Killing client %s", 
+		  silc_id_render(remote_client->id, SILC_ID_CLIENT)));
+
+  /* Send the KILL notify packets. First send it to the channel, then
+     to our primary router and then directly to the client who is being
+     killed right now. */
+
+  killed = silc_id_payload_encode(remote_client->id, SILC_ID_CLIENT);
+  killer = silc_id_payload_encode(killer_id, killer_id_type);
+
+  /* Send KILLED notify to the channels. It is not sent to the client
+     as it will be sent differently destined directly to the client and not
+     to the channel. */
+  silc_server_send_notify_on_channels(server, remote_client, 
+				      remote_client, SILC_NOTIFY_TYPE_KILLED,
+				      3, killed->data, killed->len,
+				      comment, comment ? strlen(comment) : 0,
+				      killer->data, killer->len);
+
+  /* Send KILLED notify to primary route */
+  if (!server->standalone)
+    silc_server_send_notify_killed(server, server->router->connection, TRUE,
+				   remote_client->id, comment, 
+				   killer_id, killer_id_type);
+
+  /* Send KILLED notify to the client directly */
+  if (remote_client->connection || remote_client->router)
+    silc_server_send_notify_killed(server, remote_client->connection ? 
+				   remote_client->connection : 
+				   remote_client->router->connection, FALSE,
+				   remote_client->id, comment, 
+				   killer_id, killer_id_type);
+
+  /* Remove the client from all channels. This generates new keys to the
+     channels as well. */
+  silc_server_remove_from_channels(server, NULL, remote_client, FALSE, 
+				   NULL, TRUE);
+
+  /* Remove the client entry, If it is locally connected then we will also
+     disconnect the client here */
+  if (remote_client->connection) {
+    /* Remove locally conneted client */
+    SilcSocketConnection sock = remote_client->connection;
+    silc_server_free_client_data(server, sock, remote_client, FALSE, NULL);
+    silc_server_close_connection(server, sock);
+  } else {
+    /* Update statistics */
+    server->stat.clients--;
+    server->stat.my_clients--;
+    if (server->stat.cell_clients)
+      server->stat.cell_clients--;
+    SILC_OPER_STATS_UPDATE(remote_client, server, SILC_UMODE_SERVER_OPERATOR);
+    SILC_OPER_STATS_UPDATE(remote_client, router, SILC_UMODE_ROUTER_OPERATOR);
+
+    /* Remove remote client */
+    if (!silc_idlist_del_client(server->global_list, remote_client)) {
+      /* Remove this client from watcher list if it is */
+      silc_server_del_from_watcher_list(server, remote_client);
+      silc_idlist_del_client(server->local_list, remote_client);  
+    }
+  }
+
+  silc_buffer_free(killer);
+  silc_buffer_free(killed);
+}
+
+typedef struct {
+  SilcServer server;
+  SilcClientEntry client;
+  SilcNotifyType notify;
+  const char *new_nick;
+} WatcherNotifyContext;
+
+static void 
+silc_server_check_watcher_list_foreach(void *key, void *context, 
+				       void *user_context)
+{
+  WatcherNotifyContext *notify = user_context;
+  SilcClientEntry entry = context;
+  SilcSocketConnection sock;
+
+  if (entry == notify->client)
+    return;
+
+  sock = silc_server_get_client_route(notify->server, NULL, 0, entry->id,
+				      NULL, NULL);
+  if (sock) {
+    SILC_LOG_DEBUG(("Sending WATCH notify to %s",
+		    silc_id_render(entry->id, SILC_ID_CLIENT)));
+
+    /* Send the WATCH notify */
+    silc_server_send_notify_watch(notify->server, sock, entry, 
+				  notify->client, 
+				  notify->new_nick ? notify->new_nick :
+				  (const char *)notify->client->nickname, 
+				  notify->notify);
+  }
+}
+
+/* This function checks whether the `client' nickname is being watched
+   by someone, and notifies the watcher of the notify change of notify
+   type indicated by `notify'. */
+
+bool silc_server_check_watcher_list(SilcServer server,
+				    SilcClientEntry client,
+				    const char *new_nick,
+				    SilcNotifyType notify)
+{
+  unsigned char hash[16];
+  WatcherNotifyContext n;
+
+  SILC_LOG_DEBUG(("Start"));
+
+  /* If the watching is rejected by the client do nothing */
+  if (client->mode & SILC_UMODE_REJECT_WATCHING)
+    return FALSE;
+
+  /* Make hash from the nick, or take it from Client ID */
+  if (client->nickname) {
+    char nick[128 + 1];
+    memset(nick, 0, sizeof(nick));
+    silc_to_lower(client->nickname, nick, sizeof(nick) - 1);
+    silc_hash_make(server->md5hash, nick, strlen(nick), hash);
+  } else {
+    memset(hash, 0, sizeof(hash));
+    memcpy(hash, client->id->hash, sizeof(client->id->hash));
+  }
+
+  n.server = server;
+  n.client = client;
+  n.new_nick = new_nick;
+  n.notify = notify;
+
+  /* Send notify to all watchers */
+  silc_hash_table_find_foreach(server->watcher_list, hash,
+			       silc_server_check_watcher_list_foreach, &n);
+
+  return TRUE;
+}
+
+/* Remove the `client' from watcher list. After calling this the `client'
+   is not watching any nicknames. */
+
+bool silc_server_del_from_watcher_list(SilcServer server,
+				       SilcClientEntry client)
+{
+  SilcHashTableList htl;
+  void *key;
+  SilcClientEntry entry;
+  bool found = FALSE;
+
+  silc_hash_table_list(server->watcher_list, &htl);
+  while (silc_hash_table_get(&htl, &key, (void **)&entry)) {
+    if (entry == client) {
+      silc_hash_table_del_by_context(server->watcher_list, key, client);
+
+      SILC_LOG_DEBUG(("Removing %s from WATCH list",
+		      silc_id_render(client->id, SILC_ID_CLIENT)));
+
+      /* Now check whether there still exists entries with this key, if not
+	 then free the key to not leak memory. */
+      if (!silc_hash_table_find(server->watcher_list, key, NULL, NULL))
+	silc_free(key);
+
+      found = TRUE;
+    }
+  }
+  silc_hash_table_list_reset(&htl);
+
+  return found;
+}
+
+/* Force the client indicated by `chl' to change the channel user mode
+   on channel indicated by `channel' to `forced_mode'. */
+
+bool silc_server_force_cumode_change(SilcServer server,
+				     SilcSocketConnection sock,
+				     SilcChannelEntry channel,
+				     SilcChannelClientEntry chl,
+				     SilcUInt32 forced_mode)
+{
+  SilcBuffer idp1, idp2;
+  unsigned char cumode[4];
+
+  SILC_LOG_DEBUG(("Start"));
+
+  if (sock)
+    silc_server_send_notify_cumode(server, sock, FALSE, channel, forced_mode,
+				   server->id, SILC_ID_SERVER,
+				   chl->client->id, NULL);
+
+  idp1 = silc_id_payload_encode(server->id, SILC_ID_SERVER);
+  idp2 = silc_id_payload_encode(chl->client->id, SILC_ID_CLIENT);
+  SILC_PUT32_MSB(forced_mode, cumode);
+  silc_server_send_notify_to_channel(server, sock, channel, FALSE,
+				     SILC_NOTIFY_TYPE_CUMODE_CHANGE,
+				     3, idp1->data, idp1->len,
+				     cumode, sizeof(cumode),
+				     idp2->data, idp2->len);
+  silc_buffer_free(idp1);
+  silc_buffer_free(idp2);
+
+  return TRUE;
+}
