@@ -23,7 +23,7 @@
  * This is the main program for the SILC daemon. This parses command
  * line arguments and creates the server object.
  */
-/* $Id: silcd.c,v 1.45 2002/04/17 20:10:33 johnny Exp $ */
+/* $Id: silcd.c,v 1.47 2002/06/16 20:17:55 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -245,9 +245,8 @@ static void signal_handler(int sig)
 SILC_TASK_CALLBACK(got_hup)
 {
   /* First, reset all log files (they might have been deleted) */
-  /* XXX this may be redundant with the silc_server_config_setlogfiles() call.
-   * merge these two with the appropriate checking. */
   silc_log_reset_all();
+
   /* Rehash the configuration file */
   silc_server_rehash(silcd);
 }
@@ -259,6 +258,58 @@ SILC_TASK_CALLBACK(stop_server)
   silc_schedule_stop(silcd->schedule);
 }
 
+/* Dump server statistics into a file into /tmp directory */
+
+SILC_TASK_CALLBACK(dump_stats)
+{
+  FILE *fdd;
+  char filename[256];
+
+  memset(filename, 0, sizeof(filename));
+  snprintf(filename, sizeof(filename) - 1, "/tmp/silcd.%d.stats", getpid());
+  fdd = fopen(filename, "w+");
+  if (!fdd)
+    return;
+
+#define STAT_OUTPUT(fmt, stat) fprintf(fdd, fmt "\n", (int)stat);
+
+  fprintf(fdd, "SILC Server %s Statistics\n\n", silcd->server_name);
+  fprintf(fdd, "Local Stats:\n");
+  STAT_OUTPUT("  My clients              : %d", silcd->stat.my_clients);
+  STAT_OUTPUT("  My servers              : %d", silcd->stat.my_servers);
+  STAT_OUTPUT("  My routers              : %d", silcd->stat.my_routers);
+  STAT_OUTPUT("  My channels             : %d", silcd->stat.my_channels);
+  STAT_OUTPUT("  My joined users         : %d", silcd->stat.my_chanclients);
+  STAT_OUTPUT("  My aways                : %d", silcd->stat.my_aways);
+  STAT_OUTPUT("  My detached clients     : %d", silcd->stat.my_detached);
+  STAT_OUTPUT("  My server operators     : %d", silcd->stat.my_server_ops);
+  STAT_OUTPUT("  My router operators     : %d", silcd->stat.my_router_ops);
+  fprintf(fdd, "\nGlobal Stats:\n");
+  STAT_OUTPUT("  Cell clients            : %d", silcd->stat.cell_clients);
+  STAT_OUTPUT("  Cell servers            : %d", silcd->stat.cell_servers);
+  STAT_OUTPUT("  Cell channels           : %d", silcd->stat.cell_channels);
+  STAT_OUTPUT("  Cell joined users       : %d", silcd->stat.cell_chanclients);
+  STAT_OUTPUT("  All clients             : %d", silcd->stat.clients);
+  STAT_OUTPUT("  All servers             : %d", silcd->stat.servers);
+  STAT_OUTPUT("  All routers             : %d", silcd->stat.routers);
+  STAT_OUTPUT("  All channels            : %d", silcd->stat.channels);
+  STAT_OUTPUT("  All joined users        : %d", silcd->stat.chanclients);
+  STAT_OUTPUT("  All aways               : %d", silcd->stat.aways);
+  STAT_OUTPUT("  All detached clients    : %d", silcd->stat.detached);
+  STAT_OUTPUT("  All server operators    : %d", silcd->stat.server_ops);
+  STAT_OUTPUT("  All router operators    : %d", silcd->stat.router_ops);
+  fprintf(fdd, "\nGeneral Stats:\n");
+  STAT_OUTPUT("  Connection attempts     : %d", silcd->stat.conn_attempts);
+  STAT_OUTPUT("  Connection failures     : %d", silcd->stat.conn_failures);
+  STAT_OUTPUT("  Authentication attempts : %d", silcd->stat.auth_attempts);
+  STAT_OUTPUT("  Authentication failures : %d", silcd->stat.auth_failures);
+  STAT_OUTPUT("  Packets sent            : %d", silcd->stat.packets_sent);
+  STAT_OUTPUT("  Packets received        : %d", silcd->stat.packets_received);
+
+  fflush(fdd);
+  fclose(fdd);
+}
+
 /* This function should not be called directly but throught the wrapper
    macro SILC_SERVER_LOG_STDERR() */
 
@@ -398,9 +449,11 @@ int main(int argc, char **argv)
   sigaction(SIGHUP, &sa, NULL);
   sigaction(SIGTERM, &sa, NULL);
   sigaction(SIGINT, &sa, NULL);
+  sigaction(SIGUSR1, &sa, NULL);
   silc_schedule_signal_register(silcd->schedule, SIGHUP, got_hup, NULL);
   silc_schedule_signal_register(silcd->schedule, SIGTERM, stop_server, NULL);
   silc_schedule_signal_register(silcd->schedule, SIGINT, stop_server, NULL);
+  silc_schedule_signal_register(silcd->schedule, SIGUSR1, dump_stats, NULL);
 
   if (!foreground) {
     /* Before running the server, fork to background. */
@@ -509,7 +562,7 @@ silc_server_create_key_pair(char *pkcs_name, int bits, char *path,
 
   /* Save public key into file */
   key = silc_pkcs_get_public_key(pkcs, &key_len);
-  pub_key = silc_pkcs_public_key_alloc(pkcs->pkcs->name, identifier,
+  pub_key = silc_pkcs_public_key_alloc(silc_pkcs_get_name(pkcs), identifier,
 				       key, key_len);
   silc_pkcs_save_public_key(pkfile, pub_key, SILC_PKCS_FILE_PEM);
   if (ret_pub_key)
@@ -522,7 +575,8 @@ silc_server_create_key_pair(char *pkcs_name, int bits, char *path,
 
   /* Save private key into file */
   key = silc_pkcs_get_private_key(pkcs, &key_len);
-  prv_key = silc_pkcs_private_key_alloc(pkcs->pkcs->name, key, key_len);
+  prv_key = silc_pkcs_private_key_alloc(silc_pkcs_get_name(pkcs),
+					key, key_len);
   silc_pkcs_save_private_key(prvfile, prv_key, NULL, SILC_PKCS_FILE_BIN);
   if (ret_prv_key)
     *ret_prv_key = prv_key;
